import { world, system, BlockPermutation, EntityEquippableComponent, EquipmentSlot, Direction } from "@minecraft/server";

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------------------------Funkcje dodające efekty-------------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja dodająca efekt nasycenia graczowi
async function addSaturationEffect(player) {
    const effectCommand = `effect @s saturation 1 1 true`;
    await player.runCommand(effectCommand);
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------Funkcja wyrzucająca świeczkę--------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja wyrzucająca świeczkę w zależności od typu ciasta
async function dropCandleByBlockType(block, candleColor) {
    const { x, y, z } = block.location;
    const lootItem = candleColor === "candle" ? "candle" : `${candleColor}_candle`;
    const lootCommand = `loot spawn ${x} ${y + 1} ${z} loot ${lootItem}`;
    await world.getDimension("overworld").runCommand(lootCommand);
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------------Funkcje odtwarzające dźwięk---------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja odtwarzająca dźwięk zapalania świeczki na cieście
async function playIgniteCandleCakeSound(block) {
    const random = Math.random();
    const soundName = "fire.ignite";
    const { x, y, z } = block.location;
    let soundCommand;
    if (random < 0.5) {
        soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 0.9`;
    } else if (random < 0.75) {
        soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 1`;
    } else if (random < 0.9) {
        soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 1.1`;
    } else {
        soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 1.2`;
    }
    await world.getDimension("overworld").runCommand(soundCommand);
}

// Funkcja odtwarzająca dźwięk gaszenia świeczki na cieście
async function playExtinguishCandleCakeSound(block) {
    const random = Math.random();
    const soundName = "extinguish.candle";
    const { x, y, z } = block.location;
    let soundCommand;
    if (random < 0.25) {
        soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 0.8`;
    } else if (random < 0.5) {
        soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 0.9`;
    } else if (random < 0.75) {
        soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 1`;
    } else if (random < 0.9) {
        soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 1.1`;
    } else {
        soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 1.2`;
    }
    await world.getDimension("overworld").runCommand(soundCommand);
}

// Funkcja odtwarzająca dźwięk stawiania świeczki na ciasto
async function playAddCandleCakeSound(block) {
    const soundName = "cake.add_candle";
    const { x, y, z } = block.location;
    const soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1`;
    await world.getDimension("overworld").runCommand(soundCommand);
}

// Funkcja odtwarzająca dźwięk niszczenia przedmiotu
async function playBreakItemSound(block) {
    const soundName = "random.break";
    const { x, y, z } = block.location;
    const soundCommand = `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 0.9`;
    await world.getDimension("overworld").runCommand(soundCommand);
}

// Funkcja odtwarzająca losowy dźwięk jedzenia ciasta
async function playBlockSound(player) {
    const random = Math.random();
    const soundName = "random.burp";
    const { x, y, z } = player.location;
    const soundCommand = random < 0.5
        ? `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 1 0.3`
        : `playsound ${soundName} @a[r=20,x=${x},y=${y},z=${z}] ${x} ${y} ${z} 1 0.9 0.3`;
    await world.getDimension("overworld").runCommand(soundCommand);
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------Funkcja gaszenia świeczki particle-----------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
async function handleExtinguishSmoke(event) {
    const block = event.block;
    if (!block || !block.location) {
        console.error("Blok lub jego lokalizacja są niezdefiniowane");
        return;
    }
    const { x, y, z } = block.location;
    const dimension = block.dimension;
    dimension.runCommand(`particle minecraft:basic_smoke_particle ${x} ${y + 0.9} ${z}`);
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------Funkcja pomocnicza do losowania liczby całkowitej-------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
function getRandomInt(min, max) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------------Funkcja zmiany ciasta na ciasto z świeczką----------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja zmieniająca blok na odpowiedni, jeśli gracz trzyma świeczkę
async function changeBlockIfHoldingCandle(player, block) {
    const mainhand = player.getComponent(EntityEquippableComponent.componentId).getEquipmentSlot(EquipmentSlot.Mainhand);
    const item = mainhand?.getItem();
    const itemTypeId = item?.typeId;
    const candleMap = {
        "minecraft:black_candle": "black_candle",
        "minecraft:blue_candle": "blue_candle",
        "minecraft:brown_candle": "brown_candle",
        "minecraft:cyan_candle": "cyan_candle",
        "minecraft:candle": "default_candle",
        "minecraft:gray_candle": "gray_candle",
        "minecraft:green_candle": "green_candle",
        "minecraft:light_blue_candle": "light_blue_candle",
        "minecraft:light_gray_candle": "light_gray_candle",
        "minecraft:lime_candle": "lime_candle",
        "minecraft:magenta_candle": "magenta_candle",
        "minecraft:orange_candle": "orange_candle",
        "minecraft:pink_candle": "pink_candle",
        "minecraft:purple_candle": "purple_candle",
        "minecraft:red_candle": "red_candle",
        "minecraft:white_candle": "white_candle",
        "minecraft:yellow_candle": "yellow_candle",
    };
    const blockMap = {
        "more:apple_cake": "apple_cake",
        "more:carrot_cake": "carrot_cake",
        "more:chocolate_cake": "chocolate_cake",
        "more:glow_berries_cake": "glow_berries_cake",
        "more:honey_cake": "honey_cake",
        "more:melon_cake": "melon_cake",
        "more:milk_cake": "milk_cake",
        "more:pumpkin_cake": "pumpkin_cake",
        "more:sweet_berries_cake": "sweet_berries_cake"
    };
    const currentBlockType = block.permutation.type.id;
    const currentPermutation = block.permutation;
    const currentSlices = currentPermutation.getState('cake:slices');
    if (currentSlices !== 0) {
        return;
    }
    const newBlockId = blockMap[currentBlockType] && candleMap[itemTypeId]
        ? `more:${candleMap[itemTypeId]}_${blockMap[currentBlockType]}`
        : null;
    if (newBlockId) {
        block.setPermutation(BlockPermutation.resolve(newBlockId));
        await playAddCandleCakeSound(block);
        // Jeśli gracz jest w trybie survival, odejmujemy tylko jedną świeczkę
        if (player.getGameMode() === "survival," || player.getGameMode() === "adventure," && item) {
            if (item.amount && item.amount > 1) {
                item.amount -= 1;
                mainhand.setItem(item);
            } else {
                mainhand.setItem(null);
            }
        }
    }
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------------------------------Komponent change----------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:change
async function handlePlayerInteractChange(event) {
    const player = event.player;
    const block = event.block;
    await changeBlockIfHoldingCandle(player, block);
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------Komponent set_on_fire----------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:set_on_fire 
async function setBlockOnFire(event) {
    const player = event.player;
    const block = event.block;
    const mainhand = player.getComponent(EntityEquippableComponent.componentId).getEquipmentSlot(EquipmentSlot.Mainhand);
    const item = mainhand?.getItem();
    const itemTypeId = item?.typeId;
    if (itemTypeId !== "minecraft:flint_and_steel") {
        await playerInteractWithCakeCandle(event);
        return;
    }
    const currentBlockType = block.permutation.type.id;
    if (currentBlockType.includes("_candle_")) {
        const newBlockId = currentBlockType.replace("_candle_", "_candle_lit_");
        await playIgniteCandleCakeSound(block);
        block.setPermutation(BlockPermutation.resolve(newBlockId));
        console.log(`Blok ${currentBlockType} został zapalony jako ${newBlockId}`);
        const gameMode = player.getGameMode();
        if (gameMode === "survival" || gameMode === "adventure") {
            const durabilityComponent = item.getComponent("minecraft:durability");
            if (durabilityComponent) {
                const newDamage = durabilityComponent.damage + 1;
                if (newDamage >= durabilityComponent.maxDurability) {
                    mainhand.setItem(null);
                    await playBreakItemSound(block);
                    console.log("Zapalniczka została zniszczona z powodu zużycia.");
                } else {
                    durabilityComponent.damage = newDamage;
                    mainhand.setItem(item);
                }
            }
        }
    }
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------------------------Komponent put_out_candle----------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:put_out_candle
async function extinguishCandleOnCake(event) {
    const player = event.player;
    const block = event.block;
    const mainhand = player.getComponent(EntityEquippableComponent.componentId).getEquipmentSlot(EquipmentSlot.Mainhand);
    const itemTypeId = mainhand?.getItem()?.typeId;
    const hitDirection = event.face;
    const currentBlockType = block.permutation.type.id;
    const candleColors = [
        "light_blue", "light_gray", "black", "blue", "brown", "cyan", "gray",
        "green", "lime", "magenta", "orange", "pink", "purple", "red", "white",
        "yellow", "default"
    ];
    let candleColor = null;
    for (const color of candleColors) {
        if (currentBlockType.includes(`${color}_candle`)) {
            candleColor = color;
            break;
        }
    }
    if (hitDirection === Direction.Up) {
        if (currentBlockType.includes("_candle_lit_")) {
            if (itemTypeId === "minecraft:flint_and_steel") {
                return;
            } else if (!itemTypeId) {
                const newBlockId = currentBlockType.replace("_candle_lit_", "_candle_");
                block.setPermutation(BlockPermutation.resolve(newBlockId));
                await playExtinguishCandleCakeSound(block);
                await handleExtinguishSmoke(event);
                console.log("Świeczka została zgaszona.");
                return;
            }
        }
        if (itemTypeId !== "minecraft:flint_and_steel") {
            const currentPermutation = block.permutation;
            const currentSlices = currentPermutation.getState('cake:slices');
            if (currentSlices < 6) {
                const newSlices = currentSlices + 1;
                block.setPermutation(currentPermutation.withState('cake:slices', newSlices));
                if (currentSlices === 0) {
                    await dropCandleByBlockType(block, candleColor);
                }
                await addSaturationEffect(player);
                await playBlockSound(player);
            } else {
                const airPermutation = BlockPermutation.resolve("minecraft:air");
                block.setPermutation(airPermutation);
                await addSaturationEffect(player);
                await playBlockSound(player);
            }
            return;
        }
    }
    if (hitDirection === Direction.North || hitDirection === Direction.South || hitDirection === Direction.East || hitDirection === Direction.West || hitDirection === Direction.Down) {
        if (itemTypeId !== "minecraft:flint_and_steel") {
            const currentPermutation = block.permutation;
            const currentSlices = currentPermutation.getState('cake:slices');
            if (currentSlices < 6) {
                const newSlices = currentSlices + 1;
                block.setPermutation(currentPermutation.withState('cake:slices', newSlices));
                if (currentSlices === 0) {
                    await dropCandleByBlockType(block, candleColor);
                }
                await addSaturationEffect(player);
                await playBlockSound(player);
            } else {
                const airPermutation = BlockPermutation.resolve("minecraft:air");
                block.setPermutation(airPermutation);
                await addSaturationEffect(player);
                await playBlockSound(player);
            }
            return;
        }
    }
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------------------------Komponent candle_fire i candle_smoke-------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:candle_fire
async function handleCandleFire(data) {
    const block = data.block;
    if (!block || !block.location) {
        console.error("Blok lub jego lokalizacja są niezdefiniowane");
        return;
    }
    const { x, y, z } = block.location;
    const dimensionId = block.dimension && block.dimension.id ? block.dimension.id : "overworld";
    await world.getDimension(dimensionId).runCommand(`particle minecraft:candle_flame_particle ${x} ${y + 0.978} ${z}`);
}

// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:candle_smoke
async function handleCandleSmoke(data) {
    const block = data.block;
    if (!block || !block.location) {
        console.error("Blok lub jego lokalizacja są niezdefiniowane");
        return;
    }
    const { x, y, z } = block.location;
    const dimensionId = block.dimension && block.dimension.id ? block.dimension.id : "overworld";
    await world.getDimension(dimensionId).runCommand(`particle minecraft:basic_smoke_particle ${x} ${y + 0.9} ${z}`);
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------Komponent interact_candle-----------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:interact_candle
async function playerInteractWithCakeCandle(event) {
    const player = event.player;
    const block = event.block;
    const currentBlockType = block.permutation.type.id;
    const candleColors = [
        "light_blue", "light_gray", "black", "blue", "brown", "cyan", "gray",
        "green", "lime", "magenta", "orange", "pink", "purple", "red", "white",
        "yellow", "default"
    ];
    let candleColor = null;
    for (const color of candleColors) {
        if (currentBlockType.includes(`${color}_candle`)) {
            candleColor = color;
            break;
        }
    }
    if (candleColor) {
        const currentPermutation = block.permutation;
        const currentSlices = currentPermutation.getState('cake:slices');
        if (currentSlices < 6) {
            const newSlices = currentSlices + 1;
            block.setPermutation(currentPermutation.withState('cake:slices', newSlices));
            if (currentSlices === 0) {
                await dropCandleByBlockType(block, candleColor);
            }
            await addSaturationEffect(player);
            await playBlockSound(player);
        } else {
            const airPermutation = BlockPermutation.resolve("minecraft:air");
            block.setPermutation(airPermutation);
            await addSaturationEffect(player);
            await playBlockSound(player);
        }
    }
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------------Komponent interact----------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:interact
async function handlePlayerInteract(event) {
    const player = event.player;
    const block = event.block;
    const mainhand = player.getComponent(EntityEquippableComponent.componentId).getEquipmentSlot(EquipmentSlot.Mainhand);
    const itemTypeId = mainhand?.getItem()?.typeId;
    if (itemTypeId && itemTypeId.includes("candle")) {
        return;
    }
    const currentPermutation = block.permutation;
    const currentSlices = currentPermutation.getState('cake:slices');
    if (currentSlices < 6) {
        const newSlices = currentSlices + 1;
        block.setPermutation(currentPermutation.withState('cake:slices', newSlices));
        await addSaturationEffect(player);
        await playBlockSound(player);
    } else {
        const airPermutation = BlockPermutation.resolve("minecraft:air");
        block.setPermutation(airPermutation);
        await addSaturationEffect(player);
        await playBlockSound(player);
    }
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------- Rejestracja i komponent check -----------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
system.beforeEvents.startup.subscribe((event) => {
    event.blockComponentRegistry.registerCustomComponent('cake:check', {
        onTick(data) {
            if (data.checkTickCount === undefined) {
                data.checkTickCount = 100;
            }
            if (data.checkNextTick === undefined) {
                data.checkNextTick = getRandomInt(5, 100);
            }
            data.checkTickCount++;
            if (data.checkTickCount >= data.checkNextTick) {
                const block = data.block;
                if (!block || !block.location) {
                    console.error("Blok lub jego lokalizacja są niezdefiniowane");
                    return;
                }
                const blockBelow = block.dimension.getBlock({
                    x: block.location.x,
                    y: block.location.y - 1,
                    z: block.location.z
                });
                if (blockBelow && blockBelow.typeId === "minecraft:air") {
                    console.log("Pod blokiem znajduje się powietrze");
                    const { x, y, z } = block.location;
                    block.dimension.runCommand(`fill ${x} ${y} ${z} ${x} ${y} ${z} air destroy`);
                    console.log("Blok został zniszczony");
                }
                data.checkTickCount = 0;
                data.checkNextTick = getRandomInt(5, 100);
            }
        }
    });
});

/*=============================================================================================================================================================
---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------- Skrypty dla podwójnych bloków ciasta ---------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
==============================================================================================================================================================*/

// Funkcja obsługująca interakcję gracza z blokiem dla komponentu "cake:double_cake_interact"
async function handleDoubleCakePlayerInteract(event) {
    const player = event.player;
    const block = event.block;
    const mainhand = player.getComponent(EntityEquippableComponent.componentId)
        .getEquipmentSlot(EquipmentSlot.Mainhand);
    const itemTypeId = mainhand?.getItem()?.typeId;

    // Jeżeli w ręce gracza znajduje się przedmiot zawierający "candle", funkcja wychodzi
    if (itemTypeId && itemTypeId.includes("candle")) {
        return;
    }

    const currentPermutation = block.permutation;
    const currentSlices = currentPermutation.getState('cake:slices');

    if (currentSlices < 5) {
        // Zwiększamy liczbę kawałków, jeżeli nie jest osiągnięty maksymalny limit
        const newSlices = currentSlices + 1;
        block.setPermutation(currentPermutation.withState('cake:slices', newSlices));
        await addSaturationEffect(player);
        await playBlockSound(player);
    } else {
        // Mapa konwersji bloków "double cake" na odpowiadające im bloki "single cake"
        const cakeMapping = {
            "more:double_apple_cake": "more:apple_cake",
            "more:double_carrot_cake": "more:carrot_cake",
            "more:double_chocolate_cake": "more:chocolate_cake",
            "more:double_default_cake": "minecraft:cake",
            "more:double_glow_berries_cake": "more:glow_berries_cake",
            "more:double_honey_cake": "more:honey_cake",
            "more:double_melon_cake": "more:melon_cake",
            "more:double_milk_cake": "more:milk_cake",
            "more:double_pumpkin_cake": "more:pumpkin_cake",
            "more:double_sweet_berries_cake": "more:sweet_berries_cake",
            // Dodaj kolejne mapowania dla innych typów ciasta wg potrzeb
        };

        // Pobieramy aktualny identyfikator typu bloku. 
        // Upewnij się, że metoda/ właściwość 'typeId' jest dostępna w Twoim API.
        const currentTypeId = block.typeId;
        // Wybieramy typ nowego bloku – jeżeli nie ma wpisu w mapie, domyślnie przypisujemy "minecraft:cake"
        const newCakeType = cakeMapping[currentTypeId] || "minecraft:cake";

        // Ustawiamy przekształcenie dla nowego typu ciasta
        const newPermutation = BlockPermutation.resolve(newCakeType);
        block.setPermutation(newPermutation);

        await addSaturationEffect(player);
        await playBlockSound(player);
    }
}

// Funkcja zmieniająca podwójne bloki na odpowiedni, jeśli gracz trzyma świeczkę do komponentu "cake:double_cake_change" w funkcji "handleDoubleCakePlayerInteractChange"
async function changeDoubleBlockIfHoldingCandle(player, block) {
    const mainhand = player.getComponent(EntityEquippableComponent.componentId).getEquipmentSlot(EquipmentSlot.Mainhand);
    const item = mainhand?.getItem();
    const itemTypeId = item?.typeId;
    const candleMap = {
        "minecraft:black_candle": "black_candle",
        "minecraft:blue_candle": "blue_candle",
        "minecraft:brown_candle": "brown_candle",
        "minecraft:cyan_candle": "cyan_candle",
        "minecraft:candle": "default_candle",
        "minecraft:gray_candle": "gray_candle",
        "minecraft:green_candle": "green_candle",
        "minecraft:light_blue_candle": "light_blue_candle",
        "minecraft:light_gray_candle": "light_gray_candle",
        "minecraft:lime_candle": "lime_candle",
        "minecraft:magenta_candle": "magenta_candle",
        "minecraft:orange_candle": "orange_candle",
        "minecraft:pink_candle": "pink_candle",
        "minecraft:purple_candle": "purple_candle",
        "minecraft:red_candle": "red_candle",
        "minecraft:white_candle": "white_candle",
        "minecraft:yellow_candle": "yellow_candle",
    };
    const blockMap = {
        "more:double_apple_cake": "apple_cake",
        "more:double_carrot_cake": "carrot_cake",
        "more:double_chocolate_cake": "chocolate_cake",
        "more:double_default_cake": "default_cake",
        "more:double_glow_berries_cake": "glow_berries_cake",
        "more:double_honey_cake": "honey_cake",
        "more:double_melon_cake": "melon_cake",
        "more:double_milk_cake": "milk_cake",
        "more:double_pumpkin_cake": "pumpkin_cake",
        "more:double_sweet_berries_cake": "sweet_berries_cake"
    };
    const currentBlockType = block.permutation.type.id;
    const currentPermutation = block.permutation;
    const currentSlices = currentPermutation.getState('cake:slices');
    if (currentSlices !== 0) {
        return;
    }
    const newBlockId = blockMap[currentBlockType] && candleMap[itemTypeId]
        ? `more:${candleMap[itemTypeId]}_double_${blockMap[currentBlockType]}`
        : null;
    if (newBlockId) {
        block.setPermutation(BlockPermutation.resolve(newBlockId));
        await playAddCandleCakeSound(block);
        // Jeśli gracz jest w trybie survival, odejmujemy tylko jedną świeczkę
        if (player.getGameMode() === "survival" || player.getGameMode() === "adventure" && item) {
            if (item.amount && item.amount > 1) {
                item.amount -= 1;
                mainhand.setItem(item);
            } else {
                mainhand.setItem(null);
            }
        }
    }
}

// Funkcja obsługująca interakcję gracza z blokiem dla komponentu "cake:double_cake_change"
async function handleDoubleCakePlayerInteractChange(event) {
    const player = event.player;
    const block = event.block;
    await changeDoubleBlockIfHoldingCandle(player, block);
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------Komponent interact_double_cake_candle----------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:interact_double_cake_candle
async function playerInteractWithDoubleCakeCandle(event) {
    const player = event.player;
    const block = event.block;
    const currentBlockType = block.permutation.type.id;
    const candleColors = [
        "light_blue", "light_gray", "black", "blue", "brown", "cyan", "gray",
        "green", "lime", "magenta", "orange", "pink", "purple", "red", "white",
        "yellow", "default"
    ];
    let candleColor = null;
    for (const color of candleColors) {
        if (currentBlockType.includes(`${color}_candle`)) {
            candleColor = color;
            break;
        }
    }
    if (candleColor) {
        const currentPermutation = block.permutation;
        const currentSlices = currentPermutation.getState('cake:slices');
        if (currentSlices < 5) {
            const newSlices = currentSlices + 1;
            block.setPermutation(currentPermutation.withState('cake:slices', newSlices));
            if (currentSlices === 0) {
                await dropCandleByBlockType(block, candleColor);
            }
            await addSaturationEffect(player);
            await playBlockSound(player);
        } else {
            // Mapa konwersji bloków "double cake" na odpowiadające im bloki "single cake"
            const cakeMapping = {
                "more:double_apple_cake": "more:apple_cake",
                "more:double_carrot_cake": "more:carrot_cake",
                "more:double_chocolate_cake": "more:chocolate_cake",
                "more:double_default_cake": "minecraft:cake",
                "more:double_glow_berries_cake": "more:glow_berries_cake",
                "more:double_honey_cake": "more:honey_cake",
                "more:double_melon_cake": "more:melon_cake",
                "more:double_milk_cake": "more:milk_cake",
                "more:double_pumpkin_cake": "more:pumpkin_cake",
                "more:double_sweet_berries_cake": "more:sweet_berries_cake",
                // Dodaj kolejne mapowania dla innych typów ciasta wg potrzeb
            };

            // Pobieramy aktualny identyfikator typu bloku. 
            // Upewnij się, że metoda/ właściwość 'typeId' jest dostępna w Twoim API.
            const currentTypeId = block.typeId;
            // Wybieramy typ nowego bloku – jeżeli nie ma wpisu w mapie, domyślnie przypisujemy "minecraft:cake"
            const newCakeType = cakeMapping[currentTypeId] || "minecraft:cake";
            const airPermutation = BlockPermutation.resolve(newCakeType);
            block.setPermutation(airPermutation);
            await addSaturationEffect(player);
            await playBlockSound(player);
        }
    }
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------------------------Komponent double_candle_fire i double_candle_smoke-------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:double_candle_fire
async function doubleHandleCandleFire(data) {
    const block = data.block;
    if (!block || !block.location) {
        console.error("Blok lub jego lokalizacja są niezdefiniowane");
        return;
    }
    const { x, y, z } = block.location;
    const dimensionId = block.dimension && block.dimension.id ? block.dimension.id : "overworld";
    await world.getDimension(dimensionId).runCommand(`particle minecraft:candle_flame_particle ${x} ${y + 1.467} ${z}`);
}

// Funkcja obsługująca interakcję gracza z blokiem dla komponentu cake:double_candle_smoke
async function doubleHandleCandleSmoke(data) {
    const block = data.block;
    if (!block || !block.location) {
        console.error("Blok lub jego lokalizacja są niezdefiniowane");
        return;
    }
    const { x, y, z } = block.location;
    const dimensionId = block.dimension && block.dimension.id ? block.dimension.id : "overworld";
    await world.getDimension(dimensionId).runCommand(`particle minecraft:basic_smoke_particle ${x} ${y + 1.4} ${z}`);
}

/*------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------- Rejestracja komponentów ----------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/
system.beforeEvents.startup.subscribe((initEvent) => {
    initEvent.blockComponentRegistry.registerCustomComponent('cake:interact', {
        onPlayerInteract: async (event) => {
            await handlePlayerInteract(event);
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:interact_candle', {
        onPlayerInteract: async (event) => {
            await playerInteractWithCakeCandle(event);
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:set_on_fire', {
        onPlayerInteract: async (event) => {
            await setBlockOnFire(event);
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:candle_fire', {
        onTick: async (data) => {
            if (data.fireTickCount === undefined) {
                data.fireTickCount = 5;
            }
            if (data.fireNextTick === undefined) {
                data.fireNextTick = getRandomInt(5, 100);
            }
            data.fireTickCount++;
            if (data.fireTickCount >= data.fireNextTick) {
                await handleCandleFire(data);
                data.fireTickCount = 0;
                data.fireNextTick = getRandomInt(5, 100);
            }
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:candle_smoke', {
        onTick: async (data) => {
            if (data.smokeTickCount === undefined) {
                data.smokeTickCount = 4;
            }
            if (data.smokeNextTick === undefined) {
                data.smokeNextTick = getRandomInt(5, 100);
            }
            data.smokeTickCount++;
            if (data.smokeTickCount >= data.smokeNextTick) {
                await handleCandleSmoke(data);
                data.smokeTickCount = 0;
                data.smokeNextTick = getRandomInt(5, 100);
            }
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:change', {
        onPlayerInteract: async (event) => {
            await handlePlayerInteractChange(event);
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:put_out_candle', {
        onPlayerInteract: async (event) => {
            await extinguishCandleOnCake(event);
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:double_cake_interact', {
        onPlayerInteract: async (event) => {
            await handleDoubleCakePlayerInteract(event);
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:double_cake_change', {
        onPlayerInteract: async (event) => {
            await handleDoubleCakePlayerInteractChange(event);
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:interact_double_cake_candle', {
        onPlayerInteract: async (event) => {
            await playerInteractWithDoubleCakeCandle(event);
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:double_candle_fire', {
        onTick: async (data) => {
            if (data.fireTickCount === undefined) {
                data.fireTickCount = 5;
            }
            if (data.fireNextTick === undefined) {
                data.fireNextTick = getRandomInt(5, 100);
            }
            data.fireTickCount++;
            if (data.fireTickCount >= data.fireNextTick) {
                await doubleHandleCandleFire(data);
                data.fireTickCount = 0;
                data.fireNextTick = getRandomInt(5, 100);
            }
        }
    });
    initEvent.blockComponentRegistry.registerCustomComponent('cake:double_candle_smoke', {
        onTick: async (data) => {
            if (data.smokeTickCount === undefined) {
                data.smokeTickCount = 4;
            }
            if (data.smokeNextTick === undefined) {
                data.smokeNextTick = getRandomInt(5, 100);
            }
            data.smokeTickCount++;
            if (data.smokeTickCount >= data.smokeNextTick) {
                await doubleHandleCandleSmoke(data);
                data.smokeTickCount = 0;
                data.smokeNextTick = getRandomInt(5, 100);
            }
        }
    });
});